@@ -10,16 +10,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import http.client
+
 import flask
 from flask import request
-import http.client
 from oslo_serialization import jsonutils
 
 from keystone.common import json_home
 import keystone.conf
 from keystone.server import flask as ks_flask
 
-
 CONF = keystone.conf.CONF
 MEDIA_TYPE_JSON = 'application/vnd.openstack.identity-%s+json'
 _DISCOVERY_BLUEPRINT = flask.Blueprint('Discovery', __name__)
@@ -31,19 +31,20 @@ def _get_versions_list(identity_url):
         'id': 'v3.14',
         'status': 'stable',
         'updated': '2020-04-07T00:00:00Z',
-        'links': [{
-            'rel': 'self',
-            'href': identity_url,
-        }],
-        'media-types': [{
-            'base': 'application/json',
-            'type': MEDIA_TYPE_JSON % 'v3'
-        }]
+        'links': [
+            {
+                'rel': 'self',
+                'href': identity_url,
+            }
+        ],
+        'media-types': [
+            {'base': 'application/json', 'type': MEDIA_TYPE_JSON % 'v3'}
+        ],
     }
     return versions
 
 
-class MimeTypes(object):
+class MimeTypes:
     JSON = 'application/json'
     JSON_HOME = 'application/json-home'
 
@@ -53,7 +54,8 @@ def v3_mime_type_best_match():
         return MimeTypes.JSON
 
     return request.accept_mimetypes.best_match(
-        [MimeTypes.JSON, MimeTypes.JSON_HOME])
+        [MimeTypes.JSON, MimeTypes.JSON_HOME]
+    )
 
 
 @_DISCOVERY_BLUEPRINT.route('/')
@@ -63,8 +65,10 @@ def get_versions():
         # understand the JSON-Home document.
         v3_json_home = json_home.JsonHomeResources.resources()
         json_home.translate_urls(v3_json_home, '/v3')
-        return flask.Response(response=jsonutils.dumps(v3_json_home),
-                              mimetype=MimeTypes.JSON_HOME)
+        return flask.Response(
+            response=jsonutils.dumps(v3_json_home),
+            mimetype=MimeTypes.JSON_HOME,
+        )
     else:
         identity_url = '%s/' % ks_flask.base_url()
         versions = _get_versions_list(identity_url)
@@ -76,10 +80,11 @@ def get_versions():
 
         response = flask.Response(
             response=jsonutils.dumps(
-                {'versions': {
-                    'values': list(versions.values())}}),
+                {'versions': {'values': list(versions.values())}}
+            ),
             mimetype=MimeTypes.JSON,
-            status=http.client.MULTIPLE_CHOICES)
+            status=http.client.MULTIPLE_CHOICES,
+        )
         response.headers['Location'] = preferred_location
         return response
 
@@ -90,17 +95,19 @@ def get_version_v3():
         # RENDER JSON-Home form, we have a clever client who will
         # understand the JSON-Home document.
         content = json_home.JsonHomeResources.resources()
-        return flask.Response(response=jsonutils.dumps(content),
-                              mimetype=MimeTypes.JSON_HOME)
+        return flask.Response(
+            response=jsonutils.dumps(content), mimetype=MimeTypes.JSON_HOME
+        )
     else:
         identity_url = '%s/' % ks_flask.base_url()
         versions = _get_versions_list(identity_url)
         return flask.Response(
             response=jsonutils.dumps({'version': versions['v3']}),
-            mimetype=MimeTypes.JSON)
+            mimetype=MimeTypes.JSON,
+        )
 
 
-class DiscoveryAPI(object):
+class DiscoveryAPI:
     # NOTE(morgan): The Discovery Bits are so special they cannot conform to
     # Flask-RESTful-isms. We are using straight flask Blueprint(s) here so that
     # we have a lot more control over what the heck is going on. This is just
