@@ -11,12 +11,12 @@
 # under the License.
 
 import functools
+from urllib import parse
 import uuid
 
 import flask
 from oslo_log import log
 from pycadf import cadftaxonomy as taxonomy
-from urllib import parse
 
 from keystone.auth import plugins as auth_plugins
 from keystone.auth.plugins import base
@@ -52,19 +52,22 @@ class Mapped(base.AuthMethodHandler):
         """
         if 'id' in auth_payload:
             token_ref = self._get_token_ref(auth_payload)
-            response_data = handle_scoped_token(token_ref,
-                                                PROVIDERS.federation_api,
-                                                PROVIDERS.identity_api)
+            response_data = handle_scoped_token(
+                token_ref, PROVIDERS.federation_api, PROVIDERS.identity_api
+            )
         else:
-            response_data = handle_unscoped_token(auth_payload,
-                                                  PROVIDERS.resource_api,
-                                                  PROVIDERS.federation_api,
-                                                  PROVIDERS.identity_api,
-                                                  PROVIDERS.assignment_api,
-                                                  PROVIDERS.role_api)
+            response_data = handle_unscoped_token(
+                auth_payload,
+                PROVIDERS.resource_api,
+                PROVIDERS.federation_api,
+                PROVIDERS.identity_api,
+                PROVIDERS.assignment_api,
+                PROVIDERS.role_api,
+            )
 
-        return base.AuthHandlerResponse(status=True, response_body=None,
-                                        response_data=response_data)
+        return base.AuthHandlerResponse(
+            status=True, response_body=None, response_data=response_data
+        )
 
 
 def handle_scoped_token(token, federation_api, identity_api):
@@ -78,15 +81,21 @@ def handle_scoped_token(token, federation_api, identity_api):
     for group_dict in token.federated_groups:
         group_ids.append(group_dict['id'])
     send_notification = functools.partial(
-        notifications.send_saml_audit_notification, 'authenticate',
-        user_id, group_ids, identity_provider, protocol,
-        token_audit_id)
+        notifications.send_saml_audit_notification,
+        'authenticate',
+        user_id,
+        group_ids,
+        identity_provider,
+        protocol,
+        token_audit_id,
+    )
 
     utils.assert_enabled_identity_provider(federation_api, identity_provider)
 
     try:
         mapping = federation_api.get_mapping_from_idp_and_protocol(
-            identity_provider, protocol)
+            identity_provider, protocol
+        )
         utils.validate_mapped_group_ids(group_ids, mapping['id'], identity_api)
 
     except Exception:
@@ -106,11 +115,79 @@ def handle_scoped_token(token, federation_api, identity_api):
     return response_data
 
 
-def handle_unscoped_token(auth_payload, resource_api, federation_api,
-                          identity_api, assignment_api, role_api):
+def configure_project_domain(shadow_project, idp_domain_id, resource_api):
+    """Configure federated projects domain.
 
-    def validate_shadow_mapping(shadow_projects, existing_roles, idp_domain_id,
-                                idp_id):
+    We set the domain to be the default (idp_domain_id) if the project
+    from the attribute mapping comes without a domain.
+    """
+    LOG.debug('Processing domain for project: %s', shadow_project)
+    domain = shadow_project.get('domain', {"id": idp_domain_id})
+    if 'id' not in domain:
+        db_domain = resource_api.get_domain_by_name(domain['name'])
+        domain = {"id": db_domain.get('id')}
+    shadow_project['domain'] = domain
+    LOG.debug(
+        'Project [%s] domain ID was resolved to [%s]',
+        shadow_project['name'],
+        shadow_project['domain']['id'],
+    )
+
+
+def handle_projects_from_mapping(
+    shadow_projects,
+    idp_domain_id,
+    existing_roles,
+    user,
+    assignment_api,
+    resource_api,
+):
+    for shadow_project in shadow_projects:
+        configure_project_domain(shadow_project, idp_domain_id, resource_api)
+        try:
+            # Check and see if the project already exists and if it
+            # does not, try to create it.
+            project = resource_api.get_project_by_name(
+                shadow_project['name'], shadow_project['domain']['id']
+            )
+        except exception.ProjectNotFound:
+            LOG.info(
+                'Project %(project_name)s does not exist. It will be '
+                'automatically provisioning for user %(user_id)s.',
+                {
+                    'project_name': shadow_project['name'],
+                    'user_id': user['id'],
+                },
+            )
+            project_ref = {
+                'id': uuid.uuid4().hex,
+                'name': shadow_project['name'],
+                'domain_id': shadow_project['domain']['id'],
+            }
+            project = resource_api.create_project(
+                project_ref['id'], project_ref
+            )
+        shadow_roles = shadow_project['roles']
+        for shadow_role in shadow_roles:
+            assignment_api.create_grant(
+                existing_roles[shadow_role['name']]['id'],
+                user_id=user['id'],
+                project_id=project['id'],
+            )
+
+
+def handle_unscoped_token(
+    auth_payload,
+    resource_api,
+    federation_api,
+    identity_api,
+    assignment_api,
+    role_api,
+):
+
+    def validate_shadow_mapping(
+        shadow_projects, existing_roles, user_domain_id, idp_id
+    ):
         # Validate that the roles in the shadow mapping actually exist. If
         # they don't we should bail early before creating anything.
         for shadow_project in shadow_projects:
@@ -122,65 +199,35 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
                         'Role %s was specified in the mapping but does '
                         'not exist. All roles specified in a mapping must '
                         'exist before assignment.',
-                        shadow_role['name']
+                        shadow_role['name'],
                     )
                     # NOTE(lbragstad): The RoleNotFound exception usually
                     # expects a role_id as the parameter, but in this case we
                     # only have a name so we'll pass that instead.
                     raise exception.RoleNotFound(shadow_role['name'])
                 role = existing_roles[shadow_role['name']]
-                if (role['domain_id'] is not None and
-                        role['domain_id'] != idp_domain_id):
+                if (
+                    role['domain_id'] is not None
+                    and role['domain_id'] != user_domain_id
+                ):
                     LOG.error(
                         'Role %(role)s is a domain-specific role and '
                         'cannot be assigned within %(domain)s.',
-                        {'role': shadow_role['name'], 'domain': idp_domain_id}
+                        {
+                            'role': shadow_role['name'],
+                            'domain': user_domain_id,
+                        },
                     )
                     raise exception.DomainSpecificRoleNotWithinIdPDomain(
-                        role_name=shadow_role['name'],
-                        identity_provider=idp_id
+                        role_name=shadow_role['name'], identity_provider=idp_id
                     )
 
-    def create_projects_from_mapping(shadow_projects, idp_domain_id,
-                                     existing_roles, user, assignment_api,
-                                     resource_api):
-        for shadow_project in shadow_projects:
-            try:
-                # Check and see if the project already exists and if it
-                # does not, try to create it.
-                project = resource_api.get_project_by_name(
-                    shadow_project['name'], idp_domain_id
-                )
-            except exception.ProjectNotFound:
-                LOG.info(
-                    'Project %(project_name)s does not exist. It will be '
-                    'automatically provisioning for user %(user_id)s.',
-                    {'project_name': shadow_project['name'],
-                     'user_id': user['id']}
-                )
-                project_ref = {
-                    'id': uuid.uuid4().hex,
-                    'name': shadow_project['name'],
-                    'domain_id': idp_domain_id
-                }
-                project = resource_api.create_project(
-                    project_ref['id'],
-                    project_ref
-                )
-
-            shadow_roles = shadow_project['roles']
-            for shadow_role in shadow_roles:
-                assignment_api.create_grant(
-                    existing_roles[shadow_role['name']]['id'],
-                    user_id=user['id'],
-                    project_id=project['id']
-                )
-
     def is_ephemeral_user(mapped_properties):
         return mapped_properties['user']['type'] == utils.UserType.EPHEMERAL
 
-    def build_ephemeral_user_context(user, mapped_properties,
-                                     identity_provider, protocol):
+    def build_ephemeral_user_context(
+        user, mapped_properties, identity_provider, protocol
+    ):
         resp = {}
         resp['user_id'] = user['id']
         resp['group_ids'] = mapped_properties['group_ids']
@@ -191,8 +238,9 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
 
     def build_local_user_context(mapped_properties):
         resp = {}
-        user_info = auth_plugins.UserAuthInfo.create(mapped_properties,
-                                                     METHOD_NAME)
+        user_info = auth_plugins.UserAuthInfo.create(
+            mapped_properties, METHOD_NAME
+        )
         resp['user_id'] = user_info.user_id
 
         return resp
@@ -202,12 +250,12 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
         identity_provider = auth_payload['identity_provider']
     except KeyError:
         raise exception.ValidationError(
-            attribute='identity_provider', target='mapped')
+            attribute='identity_provider', target='mapped'
+        )
     try:
         protocol = auth_payload['protocol']
     except KeyError:
-        raise exception.ValidationError(
-            attribute='protocol', target='mapped')
+        raise exception.ValidationError(attribute='protocol', target='mapped')
 
     utils.assert_enabled_identity_provider(federation_api, identity_provider)
 
@@ -223,29 +271,36 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
     try:
         try:
             mapped_properties, mapping_id = apply_mapping_filter(
-                identity_provider, protocol, assertion, resource_api,
-                federation_api, identity_api)
+                identity_provider,
+                protocol,
+                assertion,
+                resource_api,
+                federation_api,
+                identity_api,
+            )
         except exception.ValidationError as e:
             # if mapping is either invalid or yield no valid identity,
             # it is considered a failed authentication
             raise exception.Unauthorized(e)
 
         if is_ephemeral_user(mapped_properties):
-            unique_id, display_name = (
-                get_user_unique_id_and_display_name(mapped_properties)
+            idp_domain_id = federation_api.get_idp(identity_provider)[
+                'domain_id'
+            ]
+
+            validate_and_prepare_federated_user(
+                mapped_properties, idp_domain_id, resource_api
             )
-            email = mapped_properties['user'].get('email')
+
             user = identity_api.shadow_federated_user(
                 identity_provider,
-                protocol, unique_id,
-                display_name,
-                email,
-                group_ids=mapped_properties['group_ids'])
+                protocol,
+                mapped_properties['user'],
+                group_ids=mapped_properties['group_ids'],
+            )
 
             if 'projects' in mapped_properties:
-                idp_domain_id = federation_api.get_idp(
-                    identity_provider
-                )['domain_id']
+
                 existing_roles = {
                     role['name']: role for role in role_api.list_roles()
                 }
@@ -254,25 +309,27 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
                 # mapping and what it's saying to create. If there is something
                 # wrong with how the mapping is, we should bail early before we
                 # create anything.
+
                 validate_shadow_mapping(
                     mapped_properties['projects'],
                     existing_roles,
-                    idp_domain_id,
-                    identity_provider
+                    mapped_properties['user']['domain']['id'],
+                    identity_provider,
                 )
-                create_projects_from_mapping(
+                handle_projects_from_mapping(
                     mapped_properties['projects'],
                     idp_domain_id,
                     existing_roles,
                     user,
                     assignment_api,
-                    resource_api
+                    resource_api,
                 )
 
             user_id = user['id']
             group_ids = mapped_properties['group_ids']
             response_data = build_ephemeral_user_context(
-                user, mapped_properties, identity_provider, protocol)
+                user, mapped_properties, identity_provider, protocol
+            )
         else:
             response_data = build_local_user_context(mapped_properties)
 
@@ -281,19 +338,27 @@ def handle_unscoped_token(auth_payload, resource_api, federation_api,
         # send off failed authentication notification, raise the exception
         # after sending the notification
         outcome = taxonomy.OUTCOME_FAILURE
-        notifications.send_saml_audit_notification('authenticate',
-                                                   user_id, group_ids,
-                                                   identity_provider,
-                                                   protocol, token_id,
-                                                   outcome)
+        notifications.send_saml_audit_notification(
+            'authenticate',
+            user_id,
+            group_ids,
+            identity_provider,
+            protocol,
+            token_id,
+            outcome,
+        )
         raise
     else:
         outcome = taxonomy.OUTCOME_SUCCESS
-        notifications.send_saml_audit_notification('authenticate',
-                                                   user_id, group_ids,
-                                                   identity_provider,
-                                                   protocol, token_id,
-                                                   outcome)
+        notifications.send_saml_audit_notification(
+            'authenticate',
+            user_id,
+            group_ids,
+            identity_provider,
+            protocol,
+            token_id,
+            outcome,
+        )
 
     return response_data
 
@@ -303,13 +368,20 @@ def extract_assertion_data():
     return assertion
 
 
-def apply_mapping_filter(identity_provider, protocol, assertion,
-                         resource_api, federation_api, identity_api):
+def apply_mapping_filter(
+    identity_provider,
+    protocol,
+    assertion,
+    resource_api,
+    federation_api,
+    identity_api,
+):
     idp = federation_api.get_idp(identity_provider)
     utils.validate_idp(idp, protocol, assertion)
 
     mapped_properties, mapping_id = federation_api.evaluate(
-        identity_provider, protocol, assertion)
+        identity_provider, protocol, assertion
+    )
 
     # NOTE(marek-denis): We update group_ids only here to avoid fetching
     # groups identified by name/domain twice.
@@ -321,13 +393,19 @@ def apply_mapping_filter(identity_provider, protocol, assertion,
     utils.validate_mapped_group_ids(group_ids, mapping_id, identity_api)
     group_ids.extend(
         utils.transform_to_group_ids(
-            mapped_properties['group_names'], mapping_id,
-            identity_api, resource_api))
+            mapped_properties['group_names'],
+            mapping_id,
+            identity_api,
+            resource_api,
+        )
+    )
     mapped_properties['group_ids'] = list(set(group_ids))
     return mapped_properties, mapping_id
 
 
-def get_user_unique_id_and_display_name(mapped_properties):
+def validate_and_prepare_federated_user(
+    mapped_properties, idp_domain_id, resource_api
+):
     """Setup federated username.
 
     Function covers all the cases for properly setting user id, a primary
@@ -343,9 +421,18 @@ def get_user_unique_id_and_display_name(mapped_properties):
     3) If user_id is not set and user_name is, set user_id as url safe version
        of user_name.
 
+    Furthermore, we set the IdP as the user domain, if the user definition
+    does not come with a domain definition.
+
     :param mapped_properties: Properties issued by a RuleProcessor.
     :type: dictionary
 
+    :param idp_domain_id: The domain ID of the IdP registered in OpenStack.
+    :type: string
+
+    :param resource_api: The resource API used to access the database layer.
+    :type: object
+
     :raises keystone.exception.Unauthorized: If neither `user_name` nor
         `user_id` is set.
     :returns: tuple with user identification
@@ -358,9 +445,11 @@ def get_user_unique_id_and_display_name(mapped_properties):
     user_name = user.get('name') or flask.request.remote_user
 
     if not any([user_id, user_name]):
-        msg = _("Could not map user while setting ephemeral user identity. "
-                "Either mapping rules must specify user id/name or "
-                "REMOTE_USER environment variable must be set.")
+        msg = _(
+            "Could not map user while setting ephemeral user identity. "
+            "Either mapping rules must specify user id/name or "
+            "REMOTE_USER environment variable must be set."
+        )
         raise exception.Unauthorized(msg)
 
     elif not user_name:
@@ -372,4 +461,16 @@ def get_user_unique_id_and_display_name(mapped_properties):
     if user_name:
         user['name'] = user_name
     user['id'] = parse.quote(user_id)
-    return (user['id'], user['name'])
+
+    LOG.debug('Processing domain for federated user: %s', user)
+    domain = user.get('domain', {"id": idp_domain_id})
+    if 'id' not in domain:
+        db_domain = resource_api.get_domain_by_name(domain['name'])
+        domain = {"id": db_domain.get('id')}
+
+    user['domain'] = domain
+    LOG.debug(
+        'User [%s] domain ID was resolved to [%s]',
+        user['name'],
+        user['domain']['id'],
+    )
