@@ -10,14 +10,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_db import api as oslo_db_api
 import sqlalchemy
 
 from keystone.common import sql
 from keystone.models import revoke_model
 from keystone.revoke.backends import base
 
-from oslo_db import api as oslo_db_api
-
 
 class RevocationEvent(sql.ModelBase, sql.ModelDictMixin):
     __tablename__ = 'revocation_event'
@@ -39,12 +38,21 @@ class RevocationEvent(sql.ModelBase, sql.ModelDictMixin):
     audit_id = sql.Column(sql.String(32))
     audit_chain_id = sql.Column(sql.String(32))
     __table_args__ = (
-        sql.Index('ix_revocation_event_project_id_issued_before', 'project_id',
-                  'issued_before'),
-        sql.Index('ix_revocation_event_user_id_issued_before', 'user_id',
-                  'issued_before'),
-        sql.Index('ix_revocation_event_audit_id_issued_before',
-                  'audit_id', 'issued_before'),
+        sql.Index(
+            'ix_revocation_event_project_id_issued_before',
+            'project_id',
+            'issued_before',
+        ),
+        sql.Index(
+            'ix_revocation_event_user_id_issued_before',
+            'user_id',
+            'issued_before',
+        ),
+        sql.Index(
+            'ix_revocation_event_audit_id_issued_before',
+            'audit_id',
+            'issued_before',
+        ),
     )
 
 
@@ -77,8 +85,9 @@ class Revoke(base.RevokeDriverBase):
                 query = session.query(RevocationEvent.id)
                 query = query.filter(RevocationEvent.revoked_at < oldest)
                 query = query.limit(batch_size).subquery()
-                delete_query = (session.query(RevocationEvent).
-                                filter(RevocationEvent.id.in_(query)))
+                delete_query = session.query(RevocationEvent).filter(
+                    RevocationEvent.id.in_(query)
+                )
                 while True:
                     rowcount = delete_query.delete(synchronize_session=False)
                     if rowcount == 0:
@@ -93,10 +102,12 @@ class Revoke(base.RevokeDriverBase):
     def _list_token_events(self, token):
         with sql.session_for_read() as session:
             query = session.query(RevocationEvent).filter(
-                RevocationEvent.issued_before >= token['issued_at'])
+                RevocationEvent.issued_before >= token['issued_at']
+            )
             user = [RevocationEvent.user_id.is_(None)]
             proj = [RevocationEvent.project_id.is_(None)]
             audit = [RevocationEvent.audit_id.is_(None)]
+            trust = [RevocationEvent.trust_id.is_(None)]
             if token['user_id']:
                 user.append(RevocationEvent.user_id == token['user_id'])
             if token['trustor_id']:
@@ -107,16 +118,24 @@ class Revoke(base.RevokeDriverBase):
                 proj.append(RevocationEvent.project_id == token['project_id'])
             if token['audit_id']:
                 audit.append(RevocationEvent.audit_id == token['audit_id'])
-            query = query.filter(sqlalchemy.and_(sqlalchemy.or_(*user),
-                                                 sqlalchemy.or_(*proj),
-                                                 sqlalchemy.or_(*audit)))
+            if token['trust_id']:
+                trust.append(RevocationEvent.trust_id == token['trust_id'])
+            query = query.filter(
+                sqlalchemy.and_(
+                    sqlalchemy.or_(*user),
+                    sqlalchemy.or_(*proj),
+                    sqlalchemy.or_(*audit),
+                    sqlalchemy.or_(*trust),
+                )
+            )
             events = [revoke_model.RevokeEvent(**e.to_dict()) for e in query]
             return events
 
     def _list_last_fetch_events(self, last_fetch=None):
         with sql.session_for_read() as session:
             query = session.query(RevocationEvent).order_by(
-                RevocationEvent.revoked_at)
+                RevocationEvent.revoked_at
+            )
 
             if last_fetch:
                 query = query.filter(RevocationEvent.revoked_at > last_fetch)
