@@ -19,12 +19,12 @@ _role_properties = {
         'type': 'object',
         'properties': {
             'id': parameter_types.id_string,
-            'name': parameter_types.name
+            'name': parameter_types.name,
         },
         'minProperties': 1,
         'maxProperties': 1,
-        'additionalProperties': False
-    }
+        'additionalProperties': False,
+    },
 }
 
 _trust_properties = {
@@ -35,26 +35,16 @@ _trust_properties = {
     'trustee_user_id': parameter_types.external_id_string,
     'impersonation': parameter_types.boolean,
     'project_id': validation.nullable(parameter_types.id_string),
-    'remaining_uses': {
-        'type': ['integer', 'null'],
-        'minimum': 1
-    },
-    'expires_at': {
-        'type': ['null', 'string']
-    },
-    'allow_redelegation': {
-        'type': ['boolean', 'null']
-    },
-    'redelegation_count': {
-        'type': ['integer', 'null'],
-        'minimum': 0
-    },
-    'roles': _role_properties
+    'remaining_uses': {'type': ['integer', 'null'], 'minimum': 1},
+    'expires_at': {'type': ['null', 'string']},
+    'allow_redelegation': {'type': ['boolean', 'null']},
+    'redelegation_count': {'type': ['integer', 'null'], 'minimum': 0},
+    'roles': _role_properties,
 }
 
 trust_create = {
     'type': 'object',
     'properties': _trust_properties,
     'required': ['trustor_user_id', 'trustee_user_id', 'impersonation'],
-    'additionalProperties': True
+    'additionalProperties': True,
 }
